@@ -33,6 +33,7 @@
 #include <libnemo-private/nemo-action.h>
 #include <libnemo-private/nemo-file.h>
 #include <libnemo-private/nemo-file-utilities.h>
+#include <libnemo-private/nemo-global-preferences.h>
 
 #include <eel/eel-gtk-extensions.h>
 
@@ -57,18 +58,29 @@ struct NemoBlankDesktopWindowDetails {
 G_DEFINE_TYPE (NemoBlankDesktopWindow, nemo_blank_desktop_window, 
                GTK_TYPE_WINDOW);
 
+static void
+customize_clicked_callback (GtkMenuItem            *item,
+                            NemoBlankDesktopWindow *window)
+{
+    g_return_if_fail (NEMO_IS_BLANK_DESKTOP_WINDOW (window));
+
+    nemo_desktop_manager_show_desktop_overlay (nemo_desktop_manager_get (),
+                                               window->details->monitor);
+}
+
 static void
 action_activated_callback (GtkMenuItem *item, NemoAction *action)
 {
     GFile *desktop_location = nemo_get_desktop_location ();
     NemoFile *desktop_file = nemo_file_get (desktop_location);
+    GtkWindow *window = g_object_get_data (G_OBJECT (item), "nemo-window");
     g_object_unref (desktop_location);
 
-    nemo_action_activate (NEMO_ACTION (action), NULL, desktop_file);
+    nemo_action_activate (NEMO_ACTION (action), NULL, desktop_file, GTK_WINDOW (window));
 }
 
 static void
-actions_changed_cb (NemoBlankDesktopWindow *window)
+reset_popup_menu (NemoBlankDesktopWindow *window)
 {
     g_clear_pointer (&window->details->popup_menu, gtk_widget_destroy);
 }
@@ -80,18 +92,20 @@ build_menu (NemoBlankDesktopWindow *window)
         return;
     }
 
+    gboolean show_customize;
     NemoActionManager *desktop_action_manager = nemo_desktop_manager_get_action_manager ();
 
     if (window->details->actions_changed_id == 0) {
         window->details->actions_changed_id = g_signal_connect_swapped (desktop_action_manager,
                                                                         "changed",
-                                                                        G_CALLBACK (actions_changed_cb),
+                                                                        G_CALLBACK (reset_popup_menu),
                                                                         window);
     }
 
+    show_customize = g_settings_get_boolean (nemo_menu_config_preferences, "desktop-menu-customize");
     GList *action_list = nemo_action_manager_list_actions (desktop_action_manager);
 
-    if (g_list_length (action_list) == 0)
+    if (g_list_length (action_list) == 0 && !show_customize)
         return;
 
     window->details->popup_menu = gtk_menu_new ();
@@ -110,8 +124,8 @@ build_menu (NemoBlankDesktopWindow *window)
     for (l = action_list; l != NULL; l = l->next) {
         action = l->data;
 
-        if (action->show_in_blank_desktop && action->dbus_satisfied) {
-            gchar *label = nemo_action_get_label (action, NULL, NULL);
+        if (action->show_in_blank_desktop && action->dbus_satisfied && action->gsettings_satisfied) {
+            gchar *label = nemo_action_get_label (action, NULL, NULL, GTK_WINDOW (window));
             item = gtk_image_menu_item_new_with_mnemonic (label);
             g_free (label);
 
@@ -127,16 +141,32 @@ build_menu (NemoBlankDesktopWindow *window)
             }
 
             gtk_widget_set_visible (item, TRUE);
+            g_object_set_data (G_OBJECT (item), "nemo-window", window);
             g_signal_connect (item, "activate", G_CALLBACK (action_activated_callback), action);
             gtk_menu_shell_append (GTK_MENU_SHELL (window->details->popup_menu), item);
         }
     }
+
+    if (!show_customize) {
+        return;
+    }
+
+    item = gtk_menu_item_new_with_label (_("Customize"));
+
+    gtk_widget_set_visible (item, TRUE);
+    g_signal_connect (item, "activate", G_CALLBACK (customize_clicked_callback), window);
+    gtk_menu_shell_append (GTK_MENU_SHELL (window->details->popup_menu), item);
 }
 
 static void
 do_popup_menu (NemoBlankDesktopWindow *window, GdkEventButton *event)
 {
     build_menu (window);
+
+    if (window->details->popup_menu == NULL) {
+        return;
+    }
+
     eel_pop_up_context_menu (GTK_MENU(window->details->popup_menu),
                              event);
 }
@@ -174,6 +204,8 @@ nemo_blank_desktop_window_dispose (GObject *obj)
         window->details->actions_changed_id = 0;
     }
 
+    g_signal_handlers_disconnect_by_func (nemo_menu_config_preferences, reset_popup_menu, window);
+
     G_OBJECT_CLASS (nemo_blank_desktop_window_parent_class)->dispose (obj);
 }
 
@@ -198,6 +230,12 @@ nemo_blank_desktop_window_constructed (GObject *obj)
 		atk_object_set_name (accessible, _("Desktop"));
 	}
 
+    /* We don't want this extra action manager unless there's actually
+     * a blank desktop in use.  If so, however, we need to initialize it early here,
+     * as it populates itself with actions asynchronously, and would not be ready
+     * in build_menu().*/
+    nemo_desktop_manager_get_action_manager ();
+
     nemo_blank_desktop_window_update_geometry (window);
 
     gtk_window_set_resizable (GTK_WINDOW (window),
@@ -209,6 +247,11 @@ nemo_blank_desktop_window_constructed (GObject *obj)
 
     g_signal_connect (GTK_WIDGET (window), "button-press-event", G_CALLBACK (on_button_press), window);
     g_signal_connect (GTK_WIDGET (window), "popup-menu", G_CALLBACK (on_popup_menu), window);
+
+    g_signal_connect_swapped (nemo_menu_config_preferences,
+                              "changed::desktop-menu-customize",
+                              G_CALLBACK (reset_popup_menu),
+                              window);
 }
 
 static void
